/*
 * Copyright 1993-1997 Globetrotter Software, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Globetrotter Software not be used
 * in advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  Globetrotter Software makes
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * GLOBETROTTER SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO
 * EVENT SHALL GLOBETROTTER SOFTWARE BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THIS SOFTWARE.
 *
 * Author:  Jim McBeath, Globetrotter Software, jimmc@globes.com
 */
/* htserv.c - functions which are dependent on what HTTP server is running
 *
 * Jim McBeath, November 18, 1993
 */

#include <stdio.h>
#include "htimp.h"

extern char *getenv();
extern char *unescape(/*char *s,int eflag*/);

extern int ImpDebug;

void
ImpProcessServerArgs(argc,argv,ci)
int argc;
char *argv[];
CommandInfo *ci;
{
	char *p;	/* path string */
	char *m;	/* request_mode string */
	char *q;	/* query string */
	char *e;

	if (ImpDebug) {
		fprintf(stderr,"Processing command line arguments\n");
#if 0
		fprintf(stderr,"Printing environment:\n");
		system("sh -c 'env 1>&2'");
#endif
	}
#ifdef OLD_HTTPD
	if (argc<2) {
		ImpFatal("No program specified");
		/* NOTREACHED */
	}
	if (argc>3) {
		ImpFatal("Too many arguments");
		/* NOTREACHED */
	}
	p = argv[1];
	if (argc>2)
		q = argv[2];
	else
		q = 0;
#else
	p = getenv("PATH_INFO");
	if (ImpDebug)
		fprintf(stderr,"PATH_INFO=%s\n",p?p:"(null)");
	m = getenv("REQUEST_METHOD");
	if (ImpDebug)
		fprintf(stderr,"REQUEST_METHOD=%s\n",m?m:"(null)");
	if (m && strcmp(m,"POST")==0) {
		char *ns = getenv("CONTENT_LENGTH");
		int n,nx,l;
		if (ImpDebug)
			fprintf(stderr,"REQUEST_METHOD = POST\n");
		if (!ns)
			n = 0;
		else {
			n = atoi(ns);
			if (n<=0) n=0;
		}
		nx = n>0?n:10000;
		q = ImpMalloc(nx+1);
		q[0]=0;
		nx = 0;
		while (nx<n) {
			/* read it all, even if multiple lines */
			char *p;
			p = fgets(q+nx,n-nx,stdin);
			if (p==0) break;	/* no more */
			nx += strlen(q+nx);
		}
		if (ImpDebug)
			fprintf(stderr,"Stdin: %s\n",q);
		l = strlen(q);
		if (l>0 && q[l-1]=='\n')
			q[--l] = 0;	/* strip off trailing newline */
		/* TBD - make sure we got it all? */
	} else /* assume GET mode */
		q = getenv("QUERY_STRING");
#endif

	/* Check out program name */
	/* We are looking for either "/s/appname" to start program, or
	 * "/t/handle" to talk to an existing program. */
	if (!p || !*p) {
		ImpFatal("No PATH_INFO");
		/* NOTREACHED */
	}
	if (*p++!='/') {
		ImpFatal("Bad path argument");
		/* NOTREACHED */
	}
	e = strchr(p,'/');
	if (!e) {
		ImpFatal("Bad path argument");
		/* NOTREACHED */
	}
	*e++ = 0;
	if (strcmp(p,"s")==0) {
		ci->mode = MODE_START;
		ci->app = e;
		if (q && q[0])
			ci->str = ImpStrSave(q);
			/* ci->str = unescape(q,0); */
	}
	else if (strcmp(p,"t")==0) {
		ci->mode = MODE_TALK;
		ci->handle = e;
		if (q && q[0])
			ci->str = ImpStrSave(q);
			/* ci->str = unescape(q,0); */
	}
	else if (strcmp(p,"q")==0) {
		ci->mode = MODE_QUIT;
		ci->handle = e;
	}
	else {
		ImpFatal("Bad mode in path argument");
		/* NOTREACHED */
	}
	if (ci->str) {
		/* Count the number of lines in the string */
		ci->str_numlines = 1;
#if 0
		p = strchr(ci->str,'\n');
		while (p) {
			ci->str_numlines++;
			p = strchr(p+1,'\n');
		}
#endif
	} else
		ci->str_numlines = 0;
}

char *
ImpGetServerRoot()
{
	char *sr;

/* Used to be available in old httpd; not generally available now,
 * so it will typically use the built-in default; but you could set
 * SERVER_ROOT in a script that calls us if you need to.
 */
	sr = getenv("SERVER_ROOT");
	if (sr)
		return sr;
	return DEFAULT_SERVER_ROOT;
}

char *
ImpGetServerNameAndPort()
{
	char *sv;
#ifndef OLD_HTTP
	char *sp;
	static char *buf=0;
	int l;
#endif

#ifndef OLD_HTTP
	if (buf)
		return buf;
#endif
	sv = getenv("SERVER_NAME");
#ifdef OLD_HTTP	/* in old httpd, SERVER_NAME included the port number */
	if (sv)
		return sv;
#else
	sp = getenv("SERVER_PORT");
	if (sv) {
		if (!sp || strcmp(sp,"80")==0)
			return sv;	/* uses default port */
		l = strlen(sv) + strlen(sp) + 2;
		buf = ImpMalloc(l);
		sprintf(buf,"%s:%s",sv,sp);
		return buf;
	}
#endif
	return "localhost";	/* only needed for crude testing */
}

char *
ImpGetScriptName()
{
	char *sv;

/* This was not available in old httpd, so generally you would have set
 * the required value in the htimp.cf file.
 */
	sv = getenv("SCRIPT_NAME");
	if (sv)
		return sv;
	return DEFAULT_SCRIPT_FULL_NAME;
}

/* end */
